//----------------------------------------------------------------------------
//
// File:        fileio.cpp
// Date:        19-Sep-2000
// Programmer:  Marc Rousseau
//
// Description: This file contains starup code for Linux/SDL
//
// Copyright (c) 2000-2001 Marc Rousseau, All Rights Reserved.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
//
// Revision History:
//
//----------------------------------------------------------------------------

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#if ! defined ( __WIN32__ )
#include <unistd.h>
#endif
#include "common.hpp"
#include "logger.hpp"
#include "fileio.hpp"

DBG_REGISTER ( __FILE__ );

#if defined ( __OS2__ ) || defined ( __WIN32__ )
    #include <direct.h>
    #define MKDIR(x,y) mkdir ( x )
    const char FILE_SEPERATOR = '\\';
    const char *COMMON_PATH = "";
#else
    #define MKDIR(x,y) mkdir ( x, y )
    const char FILE_SEPERATOR = '/';
    const char *COMMON_PATH = "/usr/local/share/games/ti99sim";
#endif

static char home [ 256 ] = ".";

static int CreateHomePath ()
{
    FUNCTION_ENTRY ( NULL, "CreateHomePath", true );

    const char *ptr = getenv ( "HOME" );

    if ( ptr != NULL ) {
        sprintf ( home, "%s%c.ti99sim", ptr, FILE_SEPERATOR );
        MKDIR ( home, 0775 );
    }

    return 0;
}

static int  x = CreateHomePath ();

const char *HOME_PATH = home;

bool IsWriteable ( const char *filename )
{
    bool retVal = false;

    struct stat info;
    if ( stat ( filename, &info ) == 0 ) {
#if defined ( __WIN32__ )
        if ( info.st_mode & S_IWRITE ) retVal = true;
#else
        if ( getuid () == info.st_uid ) {
            if ( info.st_mode & S_IWUSR ) retVal = true;
        } else if ( getgid () == info.st_gid ) {
            if ( info.st_mode & S_IWGRP ) retVal = true;
        } else {
            if ( info.st_mode & S_IWOTH ) retVal = true;
        }
#endif
    } else {
        // TBD: Check write permissions to the directory
        retVal = true;
    }

    return retVal;
}

static bool TryPath ( const char *path, const char *filename )
{
    FUNCTION_ENTRY ( NULL, "TryPath", true );

    char buffer [256];

    sprintf ( buffer, "%s%c%s", path, FILE_SEPERATOR, filename );

    TRACE ( "Name: " << buffer );

    FILE *file = fopen ( buffer, "rb" );
    if ( file ) fclose ( file );

    return ( file == NULL ) ? false : true;
}

const char *LocateFile ( const char *path, const char *filename )
{
    FUNCTION_ENTRY ( NULL, "LocateFile", true );

    static char buffer [256];
    static char fullname[256];

    if ( filename == NULL ) return NULL;

    if ( path == NULL ) {
        path = ".";
        strcpy ( fullname, filename );
    } else {
        sprintf ( fullname, "%s%c%s", path, FILE_SEPERATOR, filename );
    }

    // If we were given an absolute path, see if it's valid or not
    if ( filename [0] == FILE_SEPERATOR ) {
        FILE *file = fopen ( filename, "rb" );
        if ( file ) fclose ( file );
        return ( file == NULL ) ? NULL : filename;
    }

    const char *pPath = NULL;
    const char *pFile = NULL;

    // Try: CWD, CWD/path, ~/.ti99sim/path, /usr/.../ti99sim/path
    if (( TryPath ( pPath = ".", pFile = filename )         == true ) ||
        ( TryPath ( pPath = ".", pFile = fullname )         == true ) ||
        ( TryPath ( pPath = home, pFile = fullname )        == true ) ||
        ( TryPath ( pPath = COMMON_PATH, pFile = fullname ) == true )) {
        sprintf ( buffer, "%s%c%s", pPath, FILE_SEPERATOR, pFile );
        return buffer;
    }

    return NULL;
}
